/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "interfacesconfigimpl.h"

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#ifdef HAVE_KSNMP
# include "loadremote.h"
#endif

#ifdef HAVE_STDINT_H
# include <stdint.h>
#elif defined HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include "loadlocal.h"

#include <kglobal.h>
#include <kiconloader.h>
#include <kcombobox.h>
#include <kpushbutton.h>
#include <klineedit.h>
#include <kdebug.h>

/**
 * @brief Commodity function to get the icon for an interface
 * @param local If true the interface will be considered as local, so the name
 *              will be checked to select the icon. If false the interface will
 *              be considered remote, and then the snmptype parameter will be
 *              checked instead.
 * @param ifname Interface name for local interfaces, could be null for remote
 *               ones.
 * @param snmptype ifType value for the remote interface. Use 0 for local
 *                 interfaces.
 *
 * This function gets th right icon for the given interface, based on the name
 * (if local) or on the SNMP ifType value (if remote).
 */
static QPixmap getIconForIF(bool local, const QString &ifname = QString::null, uint32_t snmptype = 0)
{
	static KIconLoader ifloader("knetloadapplet");
	
	if ( local )
	{
		// On BSD systems loopback interfaces has number (lo0, lo1...)
		if ( ifname.find("lo") == 0 )
			return ifloader.loadIcon("devlo", KIcon::User);
#ifdef Q_OS_LINUX
		else if ( ifname.find("eth") == 0 )
			return ifloader.loadIcon("deveth", KIcon::User);
#endif
#ifdef Q_OS_BSD4
// On BSD systems the interfaces are named after the chipset, ed is the general name
		else if ( ifname.find("ed") == 0 )
			return ifloader.loadIcon("deveth", KIcon::User);
#endif
		else if ( ifname.find("ppp") == 0 )
			return ifloader.loadIcon("devppp", KIcon::User);
		else if ( ifname.find("ippp") == 0 )
			return ifloader.loadIcon("devippp", KIcon::User);
	} else {
		switch(snmptype)
		{
		case 6:
		case 7:
		case 8:
		case 9:
		case 26:
			return ifloader.loadIcon("deveth", KIcon::User);
		case 20:
		case 21:
			return ifloader.loadIcon("devippp", KIcon::User);
		case 22:
		case 23:
			return ifloader.loadIcon("devppp", KIcon::User);
		case 24:
			return ifloader.loadIcon("devlo", KIcon::User);
		}
	}
	
	return ifloader.loadIcon("devother", KIcon::User);
}

InterfacesConfigImpl::InterfacesConfigImpl(QWidget *parent, const char *name)
    :InterfacesConfig(parent, name)
{
#ifndef HAVE_KSNMP
	kcfg_Type->setEnabled(false);
#endif
	
	btnRefresh->setPixmap( KGlobal::iconLoader()->loadIcon("reload", KIcon::Small) );
	
	enableFields();
	reloadIFs();
}

void InterfacesConfigImpl::reloadIFs()
{
	kcfg_Name->clear();
	if ( kcfg_Type->currentItem() == 0 ) // Local
	{
		const QStringList &ifaces = LoadLocal::loadInterfaces(true);
		QStringList::const_iterator enddev = ifaces.end();
		for(QStringList::const_iterator it = ifaces.begin(); it != enddev; it++)
			kcfg_Name->insertItem(getIconForIF(true, *it), *it);
	}
}

void InterfacesConfigImpl::enableFields()
{
	bool enabled = kcfg_Type->currentItem() == 1;
	
	kcfg_Server->setEnabled(enabled);
	kcfg_Community->setEnabled(enabled);
	kcfg_Version->setEnabled(enabled);
	
	kcfg_Name->clear();
}

#include "interfacesconfigimpl.moc"
