/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __KNETPROC_H
#define __KNETPROC_H

#include <qglobal.h>
#include <qstring.h>
#include <qobject.h>

#ifdef Q_OS_LINUX
# include <qfile.h>
#endif

class StatGraph;
class StatPopup;
class Settings;
class LoadReader;

/**
 * A class used to read raw network throughput details from the system.
 */
class KNetProc : public QObject {
Q_OBJECT
public:
	/**
	 * @brief Constructor
	 *
	 * @param in Input load graph
	 * @param out Output load graph
	 * @param device Interface to read the load for
	 * @param parent Parent Object
	 * @param name Name of the instance
	 */
	KNetProc(QWidget *parent, uint index, const char *name = 0);
	~KNetProc();

	StatGraph *graphIn()
	{ return m_in; }
	
	StatGraph *graphOut()
	{ return m_out; }

public slots:
	/**
	 * @brief Reloads the settings
	 *
	 * This function takes care of reloading the settings from the config file,
	 * changing the graphs when needed.
	 */
	void reloadSettings();
	
	/**
	 * @brief Takes the reading
	 *
	 * This function is used to avoid code redundancy in KNetLoad application
	 * and applet.
	 */
	void takeReading();
	
	/**
	 * @brief Toggle showing of the popup
	 *
	 * This function make the popup show (if hidden) or hide (if shown)
	 */
	void togglePopup();

private: 
//@{
/**
 * @name dev Device and results of readings.
 */
	Settings *m_sets;
		///< Settings instance for the device
	StatGraph *m_in, *m_out;
		///< Graphs for input and output of the interface
	StatPopup *m_popup;
		///< Popup with statistics of the interface
	
	LoadReader *m_reader;
	
	void readLoadActual();
		///< Do the actual reading
//@}
};

#endif
