/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LOADREADER_H
#define LOADREADER_H

#include <qobject.h>

static const float megabyte = 1024*1024;

/**
 * @brief Load reader class
 * @author Diego 'Flameeyes' Pettenò
 *
 * This is the base class for classes which does the real load readings.
 *
 * @sa LoadLocal for local readings
 * @sa LoadSNMP for remote readings via SNMP
*/
class LoadReader : public QObject
{
Q_OBJECT
public:
	LoadReader(QObject *parent = 0, const char *name = 0);
	~LoadReader();
	
	bool doReading();

	/**
	 * The total number of bytes that have passed through the selected
	 * device between the last reading and the previous reading.
	 *
	 * The values returned are guaranteed to be non-negative.
	 */
	unsigned long recentBytesIn() const
	{ return currIn - prevIn; }
	
	unsigned long recentBytesOut() const
	{ return currOut - prevOut; }

	/**
	 * The total amount of traffic that has passed through the selected
	 * device, measured in megabytes.  This amount is measured over the
	 * entire history of the device, not just from the last reading.
	 *
	 * See the const KNetProc::megabyte for exactly what constitutes a
	 * megabyte for our purposes.
	 */
	inline float totalMbIn() const
	{ return currIn / megabyte; }
	
	inline float totalMbOut() const
	{ return currOut / megabyte; }
protected:
	virtual bool doReadingActual() = 0;
	
	unsigned long currIn, currOut;
		///< Total traffic in bytes from the last reading.
	unsigned long prevIn, prevOut;
		///< Total traffic in bytes from the next-to-last reading.
};

#endif
