/***************************************************************************
 *                                                                         *
 *   KNetLoad is copyright (c) 1999-2000, Markus Gustavsson                *
 *                         (c) 2002, Ben Burton                            *
 *                         (c) 2004-2005, Diego Pettenò                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef NETLOADAPPLET_H
#define NETLOADAPPLET_H

#include <kpanelapplet.h>
#include <qptrlist.h>
#include <qstringlist.h>

class QResizeEvent;
class KAboutApplication;
class KConfigDialog;
class KListBox;
class KNetProc;
class Settings;

/**
 * @author Diego 'Flameeyes' Pettenò
 * @brief Applet for KNetLoad
 *
 * This class is an implementation of KNetLoad which instead of using
 * system tray icons, creates an applet to show the graphs.
 * This makes it bigger than the simple icons, because it uses all the height
 * of the panel.
 */
class NetLoadApplet : public KPanelApplet
{
Q_OBJECT
public:
	NetLoadApplet(const QString& configFile, Type t = Normal, int actions = 0,
		QWidget *parent = 0, const char *name = 0);
	~NetLoadApplet();

	/**
	 * @brief Gets the width of the applet when set the height.
	 * @param height Height the apple is resized to.
	 */
	virtual int widthForHeight(int height) const;

	/**
	 * @brief Gets the height of the applet when set the width.
	 * @param width Width the apple is resized to.
	 */
	virtual int heightForWidth(int width) const;

	/// Shows the about dialog for the applet.
	virtual void about();
	
protected slots:
	/// Shows the preferences dialog for the applet.
	void preferences();
	
	/// Reloads the settings for all the KNetProc instances
	void reloadSettings();
	
	/// Adds a monitored interface
	void interfaceAdd();
	
	/// Removes a monitored interface
	void interfaceRemove();
	
	/// Change settings for a monitored interface
	void interfaceEdit();
	
	/// Take the readings of all the KNetProc instances
	void takeReadings();
protected:
	/**
	@brief Function called when a resize occour
	
	This function resets the geometries of all the graphs in the applet.
	*/
	virtual void resizeEvent(QResizeEvent *e);
	
	/**
	@brief Delete and reloads the devices
	
	This function is used when some important settings has changed, and
	simply deletes and re-create all the KNetProc instances.
	*/
	void reloadDevices();
	
	/**
	@brief Commodity function for configure interfaces
	@param param Parameter for the Settings class (number of the interface)
	
	This function is called by interfaceAdd() and interfaceEdit() after
	they know which interface need to be configured (a newly added
	interface requires configuration).
	*/
	void configureIf(const QString &param);
private:
	KListBox *m_interfaces;
		///< View of list of interfaces to monitor in interface list widget
	QPtrList<KNetProc> m_procs;
		///< List of interfaces to monitor
	QStringList m_procGroups;
		///< List of groups for interfaces
	QTimer *m_readTimer;
		///< Timer to read the values of loads
	Settings *m_sets;
		///< Instance of the settings class
	
	KAboutApplication *m_aboutbox;
		///< About data for the applet.
	KConfigDialog *m_cfgdialog;
		///< Configuration dialog for the applet.
};

#endif
