/***************************************************************************
 *                                                                         *
 *   KCPULoad and KNetLoad are copyright (c) 1999-2000, Markus Gustavsson  *
 *                                       (c) 2002, Ben Burton              *
 *                                       (c) 2004, Diego Pettenò           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "statpopup.h"
#include "settings.h"

#include <qpainter.h>
#include <kdebug.h>
#include <klocale.h>
#include <kwin.h>

#define TEXT_EXPANSION_HORIZONTAL 10
#define TEXT_EXPANSION_VERTICAL 3

const QColor StatPopup::colorBorder(0, 0, 0);

StatPopup::StatPopup(QWidget *parent, const char *name) :
        QWidget(parent, name, Qt::WStyle_Customize | Qt::WStyle_NoBorder |
        Qt::WStyle_StaysOnTop | Qt::WDestructiveClose | Qt::WType_TopLevel)
        {
    // Window management.
    KWin::setState(winId(), NET::SkipTaskbar | NET::SkipPager);

    // TODO: Get all-desktops working, even after a hide/reshow.
    //KWin::setOnAllDesktops(winId(), true);

    // Basic variable initialisation.
    relX = relY = 0;
    isDragged = closing = false;

    // Initialise the pop-up window.
    readPopupState();
    
    // Initialise the text contents of this pop-up.
    fullReading = i18n("Inactive.");
    resizeToText();
}

StatPopup::~StatPopup() {
}

void StatPopup::readPopupState()
{
	Settings sets("");
	// Read the position.
	if ( sets.popupX() || sets.popupY() )
		move(sets.popupX(), sets.popupY());
	
	// Show the pop-up if appropriate.
	if ( sets.popupVisible() )
		show();
	else
		hide();
}

void StatPopup::savePopupState()
{
	Settings sets("");
	sets.setPopupX(x());
	sets.setPopupY(y());
	sets.setPopupVisible(isVisible());
	sets.writeConfig();
}

void StatPopup::paintEvent(QPaintEvent*) {
    QPainter p(this);

    // Draw the border.
    p.setPen(colorBorder);
    p.drawLine(0, 0, width(), 0);
    p.drawLine(0, 1, width(), 1);
    p.drawLine(0, 2, 0, height());
    p.drawLine(1, 2, 1, height());
    p.drawLine(width() - 2, 2, width() - 2, height());
    p.drawLine(width() - 1, 2, width() - 1, height());
    p.drawLine(2, height() - 2, width() - 2, height() - 2);
    p.drawLine(2, height() - 1, width() - 2, height() - 1);

    // Draw the text.
    p.setFont(font());
    p.setPen(colorGroup().foreground());
    p.drawText(rect(), AlignHCenter | AlignVCenter, fullReading);
}

void StatPopup::mousePressEvent(QMouseEvent* e) {
    if(e->button() == RightButton) {
        // Hide the pop-up.
        hide();
    } else {
        // Begin a drag operation.
        isDragged = true;
        relX = e->x();
        relY = e->y();
        repaint();
    }
}

void StatPopup::mouseMoveEvent(QMouseEvent* e) {
    // In the middle of a drag operation.
    move(e->globalX() - relX, e->globalY() - relY);
}

void StatPopup::mouseReleaseEvent(QMouseEvent* e) {
    // The end of a drag operation.
    move(e->globalX() - relX, e->globalY() - relY);
    isDragged = false;
    repaint();
}

void StatPopup::closeEvent(QCloseEvent* e) {
    // We're about to close.  Save the current state for the last time.
    savePopupState();
    closing = true;
    QWidget::closeEvent(e);
}

void StatPopup::hideEvent(QHideEvent* e) {
    // We're about to hide.  Save the current state if we're not
    // closing altogether.
    if (! closing)
        savePopupState();
    QWidget::hideEvent(e);
}

void StatPopup::showEvent(QShowEvent* e) {
    closing = false;
    // Make sure we're up-to-date and properly resized.
    resizeToText();

    QWidget::showEvent(e);
}

void StatPopup::resizeToText() {
    QSize size = fontMetrics().size(0, fullReading);
    resize(size.width() + 2 * TEXT_EXPANSION_HORIZONTAL,
        size.height() + 2 * TEXT_EXPANSION_VERTICAL);
    update();
}

#include "statpopup.moc"
